@@ -5,7 +5,7 @@
  (C) COPYRIGHT 1999 Eric Busboom <eric@softwarestudio.org>
      http://www.softwarestudio.org
 
- This program is free software; you can redistribute it and/or modify
+ This library is free software; you can redistribute it and/or modify
  it under the terms of either:
 
     The LGPL as published by the Free Software Foundation, version
@@ -13,7 +13,7 @@
 
  Or:
 
-    The Mozilla Public License Version 1.0. You may obtain a copy of
+    The Mozilla Public License Version 2.0. You may obtain a copy of
     the License at http://www.mozilla.org/MPL/
 ======================================================================*/
 
@@ -141,7 +141,7 @@ icalcomponent *icallangbind_get_next_component(icalcomponent *c, const char *com
 
 #define APPENDC(x) icalmemory_append_char(&buf, &buf_ptr, &buf_size, x);
 
-char *icallangbind_property_eval_string_r(icalproperty *prop, char *sep)
+char *icallangbind_property_eval_string_r(icalproperty *prop, const char *sep)
 {
     char tmp[25];
     size_t buf_size = 1024;
@@ -179,7 +179,7 @@ char *icallangbind_property_eval_string_r(icalproperty *prop, char *sep)
     APPENDS(", 'pid' ");
     APPENDS(sep);
     APPENDC('\'');
-    snprintf(tmp, 25, "%p", prop);
+    snprintf(tmp, 25, "%p", (void *)prop);
     APPENDS(tmp);
     APPENDC('\'');
 
@@ -269,7 +269,7 @@ char *icallangbind_property_eval_string_r(icalproperty *prop, char *sep)
     return buf;
 }
 
-const char *icallangbind_property_eval_string(icalproperty *prop, char *sep)
+const char *icallangbind_property_eval_string(icalproperty *prop, const char *sep)
 {
     char *buf;
 
