@@ -5,7 +5,7 @@
  (C) COPYRIGHT 2000, Eric Busboom <eric@softwarestudio.org>
      http://www.softwarestudio.org
 
- This program is free software; you can redistribute it and/or modify
+ This library is free software; you can redistribute it and/or modify
  it under the terms of either:
 
     The LGPL as published by the Free Software Foundation, version
@@ -13,7 +13,7 @@
 
  Or:
 
-    The Mozilla Public License Version 1.0. You may obtain a copy of
+    The Mozilla Public License Version 2.0. You may obtain a copy of
     the License at http://www.mozilla.org/MPL/
 ========================================================================*/
 
@@ -384,6 +384,10 @@ static void icalrecur_clause_name_and_value(struct icalrecur_parser *parser,
     *value = idx;
 }
 
+/* returns < 0 if a parsing problem:
+   -2 if an RSCALE rule is encountered yet we don't RSCALE support enabled
+   -1 for all other parsing problems
+*/
 static int icalrecur_add_byrules(struct icalrecur_parser *parser, short *array,
                                  int min, int size, char *vals)
 {
@@ -426,11 +430,14 @@ static int icalrecur_add_byrules(struct icalrecur_parser *parser, short *array,
 
         if (*t) {
             /* Check for leap month suffix (RSCALE only) */
-            if (icalrecurrencetype_rscale_is_supported() &&
-                array == parser->rt.by_month && strcmp(t, "L") == 0) {
-                /* The "L" suffix in a BYMONTH recur-rule-part
-                   is encoded by setting a high-order bit */
-                v |= LEAP_MONTH;
+            if (array == parser->rt.by_month && strcmp(t, "L") == 0) {
+                if (icalrecurrencetype_rscale_is_supported()) {
+                    /* The "L" suffix in a BYMONTH recur-rule-part
+                       is encoded by setting a high-order bit */
+                    v |= LEAP_MONTH;
+                } else {
+                    return -2;
+                }
             } else {
                 return -1;
             }
@@ -491,8 +498,8 @@ static int icalrecur_add_bydayrules(struct icalrecur_parser *parser,
 
     while (n != 0) {
         int sign = 1;
-        char weekno;  /* note: Novell/Groupwise sends BYDAY=255SU,
-                         so we fit in a signed char to get -1 SU for last Sun */
+        signed char weekno;  /* note: Novell/Groupwise sends BYDAY=255SU,
+                                so we fit in a signed char to get -1 SU for last Sun */
         icalrecurrencetype_weekday wd;
 
         if (i == ICAL_BY_DAY_SIZE) {
@@ -510,7 +517,7 @@ static int icalrecur_add_bydayrules(struct icalrecur_parser *parser,
         }
 
         /* Get Optional weekno */
-        weekno = (char)strtol(t, &t, 10);
+        weekno = (signed char)strtol(t, &t, 10);
         if (weekno < 0) {
             weekno = -weekno;
             sign = -1;
@@ -582,21 +589,41 @@ struct icalrecurrencetype icalrecurrencetype_from_string(const char *str)
             }
         } else if (strcasecmp(name, "FREQ") == 0) {
             parser.rt.freq = icalrecur_string_to_freq(value);
-            if (parser.rt.freq == ICAL_NO_RECURRENCE) r = -1;
-        } else if (icalrecurrencetype_rscale_is_supported() &&
-                   strcasecmp(name, "RSCALE") == 0) {
-            parser.rt.rscale = icalmemory_tmp_copy(value);
-        } else if (icalrecurrencetype_rscale_is_supported() &&
-                   strcasecmp(name, "SKIP") == 0) {
-            parser.rt.skip = icalrecur_string_to_skip(value);
-            if (parser.rt.skip == ICAL_SKIP_UNDEFINED) r = -1;
+            if (parser.rt.freq == ICAL_NO_RECURRENCE) {
+                r = -1;
+            }
+        } else if (strcasecmp(name, "RSCALE") == 0) {
+            if (icalrecurrencetype_rscale_is_supported()) {
+                parser.rt.rscale = icalmemory_strdup(value);
+            } else {
+                r = -2; // RSCALE clause name without RSCALE support
+            }
+        } else if (strcasecmp(name, "SKIP") == 0) {
+            if (icalrecurrencetype_rscale_is_supported()) {
+                parser.rt.skip = icalrecur_string_to_skip(value);
+                if (parser.rt.skip == ICAL_SKIP_UNDEFINED) {
+                    r = -1;
+                }
+            } else {
+                r = -2; // RSCALE clause name without RSCALE support
+            }
         } else if (strcasecmp(name, "COUNT") == 0) {
             parser.rt.count = atoi(value);
-            /* don't allow count to be less than 1 */
-            if (parser.rt.count < 1) r = -1;
+            if (parser.rt.count < 1) {
+                /* don't allow count to be less than 1 */
+                r = -1;
+            } else if (!icaltime_is_null_time(parser.rt.until)) {
+                /* don't allow both count and until */
+                r = -1;
+            }
         } else if (strcasecmp(name, "UNTIL") == 0) {
             parser.rt.until = icaltime_from_string(value);
-            if (icaltime_is_null_time(parser.rt.until)) r = -1;
+            if (icaltime_is_null_time(parser.rt.until)) {
+                r = -1;
+            } else if (parser.rt.count > 0) {
+               /* don't allow both count and until */
+               r = -1;
+            }
         } else if (strcasecmp(name, "INTERVAL") == 0) {
             parser.rt.interval = (short)atoi(value);
             /* don't allow an interval to be less than 1
@@ -640,7 +667,15 @@ struct icalrecurrencetype icalrecurrencetype_from_string(const char *str)
         }
 
         if (r) {
-            icalerror_set_errno(ICAL_MALFORMEDDATA_ERROR);
+            /* Note: silently ignore when we have an RSCALE rule, yet don't have RSCALE support.
+               The magic value "-2" indicates when that happens.
+            */
+            if (r != -2) {
+                icalerror_set_errno(ICAL_MALFORMEDDATA_ERROR);
+            }
+            if (parser.rt.rscale) {
+                free(parser.rt.rscale);
+            }
             icalrecurrencetype_clear(&parser.rt);
             break;
         }
@@ -695,7 +730,11 @@ char *icalrecurrencetype_as_string_r(struct icalrecurrencetype *recur)
     char temp[20];
     int i, j;
 
-    if (recur->freq == ICAL_NO_RECURRENCE) {
+    if (recur == 0 || recur->freq == ICAL_NO_RECURRENCE) {
+        return 0;
+    }
+
+    if (recur->until.year != 0 && recur->count != 0) {
         return 0;
     }
 
@@ -823,15 +862,17 @@ enum byrule
 
 struct icalrecur_iterator_impl
 {
-    struct icaltimetype dtstart;        /* Hack. Make into time_t */
-    struct icaltimetype last;   /* last time return from _iterator_next */
-    int occurrence_no;  /* number of step made on t iterator */
-    struct icalrecurrencetype rule;
+    struct icaltimetype dtstart;     /* copy of DTSTART: to fill in defaults */
+    struct icalrecurrencetype rule;  /* copy of RRULE */
+
+    struct icaltimetype rstart;      /* DTSTART in RSCALE  */
+    struct icaltimetype istart;      /* Gregorian start time for iterator */
+    struct icaltimetype last;        /* last time returned from iterator */
+    int occurrence_no;               /* number of steps made on the iterator */
 
 #if defined(HAVE_LIBICU)
-    UCalendar *greg;    /* Gregorian calendar */
-    UCalendar *rscale;  /* RSCALE calendar    */
-    struct icaltimetype rstart; /* DTSTART in RSCALE  */
+    UCalendar *greg;                 /* Gregorian calendar */
+    UCalendar *rscale;               /* RSCALE calendar    */
 #endif
 
     struct icaltimetype period_start;  /* Start date of monthly/yearly period */
@@ -926,7 +967,7 @@ static const struct expand_split_map_struct expand_map[] = {
     {ICAL_WEEKLY_RECURRENCE,   { 2, 2, 2, 2, 3, 3, 3, 1, 1 }},
     {ICAL_MONTHLY_RECURRENCE,  { 2, 2, 2, 2, 2, 3, 3, 1, 1 }},
     {ICAL_YEARLY_RECURRENCE,   { 2, 2, 2, 2, 2, 2, 2, 2, 1 }},
-    {ICAL_NO_RECURRENCE,       { 0, 0, 0, 0, 0, 0, 0, 0, 0 }}
+    {ICAL_NO_RECURRENCE,       { 0, 0, 0, 0, 0, 0, 0, 0, 0 }} //krazy:exclude=style
 };
 
 static int has_by_data(icalrecur_iterator *impl, enum byrule byrule)
@@ -949,6 +990,14 @@ static void setup_defaults(icalrecur_iterator *impl,
     }
 }
 
+/* Calculate the number of Gregorian months between 2 dates */
+static int __greg_month_diff(icaltimetype a, icaltimetype b)
+{
+    return (12 * (b.year - a.year) + (b.month - a.month));
+}
+
+static int __day_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b);
+
 #if defined(HAVE_LIBICU)
 /*
  * Callbacks for recurrence rules with RSCALE support (using ICU)
@@ -1181,7 +1230,7 @@ static struct icaltimetype occurrence_as_icaltime(icalrecur_iterator *impl,
     UCalendar *cal = impl->rscale;
     int is_leap_month = 0;
 
-    if (impl->greg && normalize) {
+    if (normalize && (impl->rscale != impl->greg)) {
         /* Convert to Gregorian date */
         UDate millis = ucal_getMillis(impl->rscale, &status);
 
@@ -1272,7 +1321,7 @@ static void increment_second(icalrecur_iterator *impl, int inc)
 static int validate_byrule(icalrecur_iterator *impl,
                            enum byrule byrule, UCalendarDateFields field,
                            short (*decode_val)(short *, unsigned),
-                           unsigned decode_flags)
+                           unsigned int decode_flags)
 {
     if (has_by_data(impl, byrule)) {
         UErrorCode status = U_ZERO_ERROR;
@@ -1292,7 +1341,7 @@ static int validate_byrule(icalrecur_iterator *impl,
     return 1;
 }
 
-static short decode_month(short *month, unsigned is_hebrew)
+static short decode_month(short *month, unsigned int is_hebrew)
 {
     if (is_hebrew && *month > 5) {  /* 5L == 0x1005 */
         /* Hebrew calendar:
@@ -1304,20 +1353,20 @@ static short decode_month(short *month, unsigned is_hebrew)
     return icalrecurrencetype_month_month(*month) - 1;  /* UCal is 0-based */
 }
 
-static short decode_day(short *day, unsigned flags)
+static short decode_day(short *day, unsigned int flags)
 {
     _unused(flags);
 
     return icalrecurrencetype_day_position(*day);
 }
 
-static int initialize_iterator(icalrecur_iterator *impl)
+static int initialize_rscale(icalrecur_iterator *impl)
 {
     struct icalrecurrencetype rule = impl->rule;
     struct icaltimetype dtstart = impl->dtstart;
     char locale[ULOC_KEYWORD_AND_VALUES_CAPACITY] = "";
     UErrorCode status = U_ZERO_ERROR;
-    UChar *tzid = NULL;
+    UChar *tzid = (UChar *) UCAL_UNKNOWN_ZONE_ID;
     short is_hebrew = 0;
 
     if (dtstart.zone) {
@@ -1325,7 +1374,7 @@ static int initialize_iterator(icalrecur_iterator *impl)
         const char *src = icaltimezone_get_tzid((icaltimezone *) dtstart.zone);
         size_t len = (strlen(src) + 1) * U_SIZEOF_UCHAR;
         tzid = icalmemory_tmp_buffer(len);
-        tzid = u_strFromUTF8Lenient(tzid, len, NULL, src, -1, &status);
+        tzid = u_strFromUTF8Lenient(tzid, (int32_t)len, NULL, src, -1, &status);
         if (U_FAILURE(status)) {
             icalerror_set_errno(ICAL_INTERNAL_ERROR);
             return 0;
@@ -1356,7 +1405,6 @@ static int initialize_iterator(icalrecur_iterator *impl)
     if (!rule.rscale) {
         /* Use Gregorian as RSCALE */
         impl->rscale = impl->greg;
-        impl->greg = NULL;
     } else {
         UEnumeration *en;
         const char *cal;
@@ -1401,7 +1449,7 @@ static int initialize_iterator(icalrecur_iterator *impl)
 
     /* Validate BY_* array values whose legal maximums differ based on RSCALE */
     if (!validate_byrule(impl, BY_MONTH, UCAL_MONTH,
-                         &decode_month, (unsigned)is_hebrew) ||
+                         &decode_month, (unsigned int)is_hebrew) ||
         !validate_byrule(impl, BY_DAY, UCAL_WEEK_OF_YEAR, &decode_day, 0) ||
         !validate_byrule(impl, BY_MONTH_DAY, UCAL_DAY_OF_MONTH, NULL, 0) ||
         !validate_byrule(impl, BY_YEAR_DAY, UCAL_DAY_OF_YEAR, NULL, 0) ||
@@ -1418,20 +1466,93 @@ static int initialize_iterator(icalrecur_iterator *impl)
     /* Get rstart (DTSTART in RSCALE) */
     impl->rstart = occurrence_as_icaltime(impl, 0);
 
-    /* Set up defaults for BY_* arrays */
-    setup_defaults(impl, BY_SECOND, impl->rstart.second);
-
-    setup_defaults(impl, BY_MINUTE, impl->rstart.minute);
-
-    setup_defaults(impl, BY_HOUR, impl->rstart.hour);
-
-    setup_defaults(impl, BY_MONTH_DAY, impl->rstart.day);
-
-    setup_defaults(impl, BY_MONTH, impl->rstart.month);
-
     return 1;
 }
 
+/* Set Gregorian date and convert to RSCALE */
+static void set_datetime(icalrecur_iterator *impl, icaltimetype date)
+{
+    UErrorCode status = U_ZERO_ERROR;
+
+    impl->last.is_date = impl->rstart.is_date;
+    impl->last.zone = impl->rstart.zone;
+
+    if (impl->rstart.is_date) {
+        ucal_setDate(impl->greg,
+                     (int32_t) date.year,
+                     (int32_t) (date.month - 1), /* UCal is 0-based */
+                     (int32_t) date.day, &status);
+    } else {
+        ucal_setDateTime(impl->greg,
+                         (int32_t) date.year,
+                         (int32_t) (date.month - 1), /* UCal is 0-based */
+                         (int32_t) date.day,
+                         (int32_t) (has_by_data(impl, BY_HOUR) ?
+                                    impl->by_ptrs[BY_HOUR][0] : impl->rstart.hour),
+                         (int32_t) (has_by_data(impl, BY_MINUTE) ?
+                                    impl->by_ptrs[BY_MINUTE][0] : impl->rstart.minute),
+                         (int32_t) (has_by_data(impl, BY_SECOND) ?
+                                    impl->by_ptrs[BY_SECOND][0] : impl->rstart.second),
+                         &status);
+    }
+
+    if (impl->rscale != impl->greg) {
+        UDate millis = ucal_getMillis(impl->greg, &status);
+        ucal_setMillis(impl->rscale, millis, &status);
+    }
+}
+
+/* Calculate the number of RSCALE months between 2 dates */
+static int month_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b)
+{
+    int diff;
+
+    if (impl->rscale == impl->greg) {
+        /* Use simple Gregorian math */
+        diff = __greg_month_diff(a, b);
+    } else if (a.year == b.year) {
+        diff = b.month - a.month;
+    } else {
+        /* Count months in each year to account for leap months */
+        UErrorCode status = U_ZERO_ERROR;
+        UDate millis;
+        int year = a.year;
+
+        /* Save current date */
+        millis = ucal_getMillis(impl->rscale, &status);
+
+        set_day_of_year(impl, 1);
+        diff = get_months_in_year(impl, year) - a.month;
+        while (++year < b.year) diff += get_months_in_year(impl, year);
+        diff += b.month;
+
+        /* Restore date */
+        ucal_setMillis(impl->rscale, millis, &status);
+    }
+
+    return diff;
+}
+
+/* Calculate the number of RSCALE days between 2 dates */
+static int day_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b)
+{
+    UErrorCode status = U_ZERO_ERROR;
+    UDate millis;
+    int diff;
+
+    /* Save current date */
+    millis = ucal_getMillis(impl->rscale, &status);
+
+    set_day_of_year(impl, 1);
+
+    diff = __day_diff(impl, a, b);
+
+    /* Restore date */
+    ucal_setMillis(impl->rscale, millis, &status);
+
+    return diff;
+}
+
 #else /* !HAVE_LIBICU */
 
 /*
@@ -1618,99 +1739,104 @@ static void __increment_month(icalrecur_iterator *impl, int inc)
 
 static void increment_monthday(icalrecur_iterator *impl, int inc)
 {
-    int i;
-
-    if (inc < 0) {
-        impl->last.day += inc;
-        icaltime_normalize(impl->last);
-        return;
-    }
-
-    for (i = 0; i < inc; i++) {
-        int days_in_month =
-            icaltime_days_in_month(impl->last.month, impl->last.year);
-
-        impl->last.day++;
-
-        if (impl->last.day > days_in_month) {
-            impl->last.day = impl->last.day - days_in_month;
-            __increment_month(impl, 1);
-        }
-    }
+    icaltime_adjust(&impl->last, inc, 0, 0, 0);
 }
 
 static void increment_hour(icalrecur_iterator *impl, int inc)
 {
-    int days;
-
-    impl->last.hour += inc;
-
-    days = impl->last.hour / 24;
-    impl->last.hour = impl->last.hour % 24;
-
-    if (days != 0) {
-        increment_monthday(impl, days);
-    }
+    icaltime_adjust(&impl->last, 0, inc, 0, 0);
 }
 
 static void increment_minute(icalrecur_iterator *impl, int inc)
 {
-    int hours;
-
-    impl->last.minute += inc;
-
-    hours = impl->last.minute / 60;
-    impl->last.minute = impl->last.minute % 60;
-
-    if (hours != 0) {
-        increment_hour(impl, hours);
-    }
+    icaltime_adjust(&impl->last, 0, 0, inc, 0);
 }
 
 static void increment_second(icalrecur_iterator *impl, int inc)
 {
-    int minutes;
-
-    impl->last.second += inc;
-
-    minutes = impl->last.second / 60;
-    impl->last.second = impl->last.second % 60;
-
-    if (minutes != 0) {
-        increment_minute(impl, minutes);
-    }
+    icaltime_adjust(&impl->last, 0, 0, 0, inc);
 }
 
-static int initialize_iterator(icalrecur_iterator *impl)
+static int initialize_rscale(icalrecur_iterator *impl)
 {
-    /* Set up defaults for BY_* arrays */
-    setup_defaults(impl, BY_SECOND, impl->dtstart.second);
-
-    setup_defaults(impl, BY_MINUTE, impl->dtstart.minute);
-
-    setup_defaults(impl, BY_HOUR, impl->dtstart.hour);
-
-    setup_defaults(impl, BY_MONTH_DAY, impl->dtstart.day);
-
-    setup_defaults(impl, BY_MONTH, impl->dtstart.month);
+    impl->rstart = impl->dtstart;
 
     return 1;
 }
 
+/* Set Gregorian date */
+static void set_datetime(icalrecur_iterator *impl, icaltimetype date)
+{
+    impl->last.year = date.year;
+    impl->last.month = date.month;
+    impl->last.day = date.day;
+    impl->last.is_date = impl->dtstart.is_date;
+    impl->last.zone = impl->dtstart.zone;
+
+    if (!impl->dtstart.is_date) {
+        impl->last.hour = has_by_data(impl, BY_HOUR) ?
+            impl->by_ptrs[BY_HOUR][0] : impl->dtstart.hour;
+        impl->last.minute = has_by_data(impl, BY_MINUTE) ?
+            impl->by_ptrs[BY_MINUTE][0] : impl->dtstart.minute;
+        impl->last.second = has_by_data(impl, BY_SECOND) ?
+            impl->by_ptrs[BY_SECOND][0] : impl->dtstart.second;
+    }
+}
+
+/* Calculate the number of Gregorian months between 2 dates */
+static int month_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b)
+{
+    _unused(impl);
+
+    return __greg_month_diff(a, b);
+}
+
+/* Calculate the number of Gregorian days between 2 dates */
+static int day_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b)
+{
+    return __day_diff(impl, a, b);
+}
+
 #endif /* HAVE_LIBICU */
 
+static int __iterator_set_start(icalrecur_iterator *impl, icaltimetype start);
 static void increment_month(icalrecur_iterator *impl);
 static int expand_month_days(icalrecur_iterator *impl, int year, int month);
 static int expand_year_days(icalrecur_iterator *impl, int year);
 static int next_yearday(icalrecur_iterator *impl,
                         void (*next_period)(icalrecur_iterator *));
 
+static void adjust_to_byday(icalrecur_iterator *impl)
+{
+    /* If there is BY_DAY data, then we need to move the initial
+       time to the start of the BY_DAY data. That is if the
+       start time is on a Wednesday, and the rule has
+       BYDAY=MO,WE,FR, move the initial time back to
+       monday. Otherwise, jumping to the next week ( jumping 7
+       days ahead ) will skip over some occurrences in the
+       second week. */
+
+    /* This depends on impl->by_ptrs[BY_DAY] being correctly sorted by
+     * day. This should probably be abstracted to make such assumption
+     * more explicit. */
+    short this_dow = (short)get_day_of_week(impl);
+    short dow = (short)(impl->by_ptrs[BY_DAY][0] - this_dow);
+
+    /* Normalize day of week around week start */
+    if (dow != 0 && this_dow < (short)impl->rule.week_start)
+        dow -= 7;
+
+    if ((this_dow < impl->by_ptrs[BY_DAY][0] && dow >= 0) || dow < 0) {
+        /* initial time is after first day of BY_DAY data */
+        increment_monthday(impl, dow);
+    }
+}
+
 icalrecur_iterator *icalrecur_iterator_new(struct icalrecurrencetype rule,
                                            struct icaltimetype dtstart)
 {
     icalrecur_iterator *impl;
     icalrecurrencetype_frequency freq = rule.freq;
-    struct icaltimetype last;
     enum byrule byrule;
 
     icalerror_clear_errno();
@@ -1742,11 +1868,8 @@ icalrecur_iterator *icalrecur_iterator_new(struct icalrecurrencetype rule,
 
     memset(impl, 0, sizeof(icalrecur_iterator));
 
-    impl->rule = rule;
-    impl->last = dtstart;
     impl->dtstart = dtstart;
-    impl->days_index = ICAL_YEARDAYS_MASK_SIZE;
-    impl->occurrence_no = 0;
+    impl->rule = rule;
 
     /* Set up convenience pointers to make the code simpler. Allows
        us to iterate through all of the BY* arrays in the rule. */
@@ -1798,106 +1921,24 @@ icalrecur_iterator *icalrecur_iterator_new(struct icalrecurrencetype rule,
         }
     }
 
-    /* Rewrite some of the rules and set up defaults to make later
-       processing easier. Primarily, it involves copying an element
-       from the start time into the corresponding BY_* array when the
-       BY_* array is empty */
-
-    if (initialize_iterator(impl) == 0) {
+    if (initialize_rscale(impl) == 0) {
         icalrecur_iterator_free(impl);
         return 0;
     }
 
-    last = occurrence_as_icaltime(impl, 0);
+    /* Set up defaults for BY_* arrays */
+    setup_defaults(impl, BY_SECOND, impl->rstart.second);
 
-    switch (impl->rule.freq) {
-    case ICAL_WEEKLY_RECURRENCE:
+    setup_defaults(impl, BY_MINUTE, impl->rstart.minute);
 
-        if (impl->by_ptrs[BY_DAY][0] == ICAL_RECURRENCE_ARRAY_MAX) {
+    setup_defaults(impl, BY_HOUR, impl->rstart.hour);
 
-            /* Weekly recurrences with no BY_DAY data should occur on the
-               same day of the week as the start time . */
-            impl->by_ptrs[BY_DAY][0] = (short)get_day_of_week(impl);
+    setup_defaults(impl, BY_MONTH_DAY, impl->rstart.day);
 
-        } else {
-            /* If there is BY_DAY data, then we need to move the initial
-               time to the start of the BY_DAY data. That is if the
-               start time is on a Wednesday, and the rule has
-               BYDAY=MO,WE,FR, move the initial time back to
-               monday. Otherwise, jumping to the next week ( jumping 7
-               days ahead ) will skip over some occurrences in the
-               second week. */
+    setup_defaults(impl, BY_MONTH, impl->rstart.month);
 
-            /* This depends on impl->by_ptrs[BY_DAY] being correctly sorted by
-             * day. This should probably be abstracted to make such assumption
-             * more explicit. */
-            short this_dow = (short)get_day_of_week(impl);
-            short dow = (short)(impl->by_ptrs[BY_DAY][0] - this_dow);
-
-            /* Normalize day of week around week start */
-            if (dow != 0 && this_dow < (short)impl->rule.week_start)
-                dow -= 7;
-
-            if ((this_dow < impl->by_ptrs[BY_DAY][0] && dow >= 0) || dow < 0) {
-                /* initial time is after first day of BY_DAY data */
-                increment_monthday(impl, dow);
-            }
-        }
-        break;
-
-    case ICAL_YEARLY_RECURRENCE:
-        /* For YEARLY rule, begin by setting up the year days array.
-           The YEARLY rules work by expanding one year at a time. */
-
-        /* Fail after hitting the year 20000 if no expanded days match */
-        while (last.year < 20000) {
-            expand_year_days(impl, last.year);
-            if (icalerrno != ICAL_NO_ERROR) {
-                icalerror_set_errno(ICAL_MALFORMEDDATA_ERROR);
-                free(impl);
-                return 0;
-            }
-            if (impl->days_index < ICAL_YEARDAYS_MASK_SIZE) {
-                break;  /* break when a matching day is found */
-            }
-            increment_year(impl, impl->rule.interval);
-            last = occurrence_as_icaltime(impl, 0);
-        }
-
-        /* Copy the first day into last. */
-        set_day_of_year(impl, impl->days_index);
-
-        break;
-
-    case ICAL_MONTHLY_RECURRENCE:
-        /* For MONTHLY rule, begin by setting up the year days array.
-           The MONTHLY rules work by expanding one month at a time. */
-
-        /* Fail after hitting the year 20000 if no expanded days match */
-        while (last.year < 20000) {
-            expand_month_days(impl, last.year, last.month);
-            if (impl->days_index < ICAL_YEARDAYS_MASK_SIZE) {
-                break;  /* break when a matching day is found */
-            }
-            increment_month(impl);
-            last = occurrence_as_icaltime(impl, 0);
-        }
-
-        /* Copy the first day into last. */
-        set_day_of_year(impl, impl->days_index);
-
-        break;
-
-    default:
-        break;
-    }
-
-    impl->last = occurrence_as_icaltime(impl, 1);
-
-    /* Fail if first instance exceeds MAX_TIME_T_YEAR */
-    if (impl->last.year > MAX_TIME_T_YEAR) {
-        icalerror_set_errno(ICAL_MALFORMEDDATA_ERROR);
-        free(impl);
+    if (!__iterator_set_start(impl, dtstart)) {
+        icalrecur_iterator_free(impl);
         return 0;
     }
 
@@ -1910,46 +1951,77 @@ void icalrecur_iterator_free(icalrecur_iterator *i)
 
 #if defined(HAVE_LIBICU)
     if (i->greg) {
+        if (i->rscale && (i->rscale != i->greg)) {
+            ucal_close(i->rscale);
+        }
+
         ucal_close(i->greg);
     }
-    if (i->rscale) {
-        ucal_close(i->rscale);
-    }
 #endif
 
     free(i);
 }
 
+/* Calculate the number of days between 2 dates */
+static int __day_diff(icalrecur_iterator *impl, icaltimetype a, icaltimetype b)
+{
+    int diff;
+
+    if (a.year == b.year) {
+        diff = get_day_of_year(impl, b.year, b.month, b.day, NULL) -
+            get_day_of_year(impl, a.year, a.month, a.day, NULL);
+    } else {
+        /* Swap a and b if a is greater than b */
+        int flipped = 0;
+        int year;
+
+        if (a.year > b.year) {
+            icaltimetype temp = a;
+
+            a = b;
+            b = temp;
+            flipped = 1;
+        }
+
+        /* Count days in each year to account for leap days/months */
+        year = a.year;
+
+        diff = get_days_in_year(impl, year) -
+            get_day_of_year(impl, a.year, a.month, a.day, NULL);
+        while (++year < b.year) diff += get_days_in_year(impl, year);
+        diff += get_day_of_year(impl, b.year, b.month, b.day, NULL);
+
+        if (flipped) {
+            /* The difference is negative because a was greater than b */
+            diff = -diff;
+        }
+    }
+
+    return diff;
+}
+
 /** Increment month is different that the other increment_* routines --
    it figures out the interval for itself, and uses BYMONTH data if
    available. */
 static void increment_month(icalrecur_iterator *impl)
 {
+    int inc = impl->rule.interval;
+
+    __increment_month(impl, inc);
+
     if (has_by_data(impl, BY_MONTH)) {
-        /* Ignore the frequency and use the byrule data */
+        struct icaltimetype this = occurrence_as_icaltime(impl, 0);
 
-        do {
-            BYMONIDX++;
+        while (this.year < 20000) {
+            for (BYMONIDX = 0;
+                 BYMONPTR[BYMONIDX] != ICAL_RECURRENCE_ARRAY_MAX; BYMONIDX++) {
 
-            if (BYMONPTR[BYMONIDX] == ICAL_RECURRENCE_ARRAY_MAX) {
-                BYMONIDX = 0;
-
-                increment_year(impl, 1);
+                if (this.month == BYMONPTR[BYMONIDX]) return;
             }
 
-        } while (!set_month(impl, BYMONPTR[BYMONIDX]));
-
-    } else {
-
-        int inc;
-
-        if (impl->rule.freq == ICAL_MONTHLY_RECURRENCE) {
-            inc = impl->rule.interval;
-        } else {
-            inc = 1;
+            __increment_month(impl, inc);
+            this = occurrence_as_icaltime(impl, 0);
         }
-
-        __increment_month(impl, inc);
     }
 }
 
@@ -2727,8 +2799,7 @@ static int check_contracting_rules(icalrecur_iterator *impl)
 
 struct icaltimetype icalrecur_iterator_next(icalrecur_iterator *impl)
 {
-    int valid = 1;
-
+    /* Quit if we reached COUNT or if last time is after the UNTIL time */
     if (!impl ||
         (impl->rule.count != 0 && impl->occurrence_no >= impl->rule.count) ||
         (!icaltime_is_null_time(impl->rule.until) &&
@@ -2736,15 +2807,17 @@ struct icaltimetype icalrecur_iterator_next(icalrecur_iterator *impl)
         return icaltime_null_time();
     }
 
-    if (impl->occurrence_no == 0 && check_contracting_rules(impl)
-        && icaltime_compare(impl->last, impl->dtstart) >= 0) {
+    /* If initial time is valid, return it */
+    if ((impl->occurrence_no == 0) &&
+        (icaltime_compare(impl->last, impl->istart) >= 0) &&
+        check_contracting_rules(impl)) {
 
         impl->occurrence_no++;
         return impl->last;
     }
 
+    /* Iterate until we get the next valid time */
     do {
-        valid = 1;
         switch (impl->rule.freq) {
 
         case ICAL_SECONDLY_RECURRENCE:
@@ -2782,25 +2855,182 @@ struct icaltimetype icalrecur_iterator_next(icalrecur_iterator *impl)
 
         impl->last = occurrence_as_icaltime(impl, 1);
 
+        /* Ignore times that are after the MAX year or the UNTIL time */
+        if (impl->last.year > MAX_TIME_T_YEAR ||
+            (!icaltime_is_null_time(impl->rule.until) &&
+             icaltime_compare(impl->last, impl->rule.until) > 0)) {
+            return icaltime_null_time();
+        }
         if (impl->last.year > MAX_TIME_T_YEAR) {
             /* HACK */
             return icaltime_null_time();
         }
 
-    } while (!check_contracting_rules(impl) ||
-             icaltime_compare(impl->last, impl->dtstart) < 0 || valid == 0);
-
-    /* Ignore null times and times that are after the until time */
-    if (!icaltime_is_null_time(impl->rule.until) &&
-        icaltime_compare(impl->last, impl->rule.until) > 0) {
-        return icaltime_null_time();
-    }
+    } while (icaltime_compare(impl->last, impl->istart) < 0 ||
+             !check_contracting_rules(impl));
 
     impl->occurrence_no++;
 
     return impl->last;
 }
 
+static int __iterator_set_start(icalrecur_iterator *impl, icaltimetype start)
+{
+    icalrecurrencetype_frequency freq = impl->rule.freq;
+    short interval = impl->rule.interval;
+    int diff;
+
+    impl->istart = start;
+    impl->occurrence_no = 0;
+    impl->days_index = ICAL_YEARDAYS_MASK_SIZE;
+
+    /* Set Gregorian start date */
+    set_datetime(impl, start);
+
+    switch (freq) {
+    case ICAL_YEARLY_RECURRENCE:
+        /* For YEARLY rule, begin by setting up the year days array.
+           The YEARLY rules work by expanding one year at a time. */
+
+        if ((interval > 1) &&
+            (diff = (impl->istart.year - impl->rstart.year) % interval)) {
+            /* Specified start year doesn't match interval -
+               bump start to first day of next year that matches interval */
+            set_day_of_year(impl, 1);
+            increment_year(impl, interval - diff);
+        }
+
+        /* Get (adjusted) start date as RSCALE date */
+        start = occurrence_as_icaltime(impl, 0);
+
+        /* Expand days array for (adjusted) start year -
+           fail after hitting the year 20000 if no expanded days match */
+        while (start.year < 20000) {
+            expand_year_days(impl, start.year);
+            if (icalerrno != ICAL_NO_ERROR) {
+                icalerror_set_errno(ICAL_MALFORMEDDATA_ERROR);
+                return 0;
+            }
+            if (impl->days_index < ICAL_YEARDAYS_MASK_SIZE) {
+                break;  /* break when a matching day is found */
+            }
+            increment_year(impl, interval);
+            start = occurrence_as_icaltime(impl, 0);
+        }
+
+        /* Copy the first day into last */
+        set_day_of_year(impl, impl->days_index);
+
+        break;
+
+    case ICAL_MONTHLY_RECURRENCE:
+        /* For MONTHLY rule, begin by setting up the year days array.
+           The MONTHLY rules work by expanding one month at a time. */
+
+        if ((interval > 1) &&
+            (diff = month_diff(impl, impl->rstart, impl->istart) % interval)) {
+            /* Specified month doesn't match interval -
+               bump start to first day of next month that matches interval */
+            increment_monthday(impl, -impl->istart.day + 1);
+            __increment_month(impl, interval - diff);
+        }
+
+        /* Get (adjusted) start date as RSCALE date */
+        start = occurrence_as_icaltime(impl, 0);
+
+        /* Expand days array for (adjusted) start month -
+           fail after hitting the year 20000 if no expanded days match */
+        while (start.year < 20000) {
+            expand_month_days(impl, start.year, start.month);
+            if (impl->days_index < ICAL_YEARDAYS_MASK_SIZE) {
+                break;  /* break when a matching day is found */
+            }
+            increment_month(impl);
+            start = occurrence_as_icaltime(impl, 0);
+        }
+
+        /* Copy the first day into last */
+        set_day_of_year(impl, impl->days_index);
+
+        break;
+
+    case ICAL_WEEKLY_RECURRENCE:
+        if (impl->by_ptrs[BY_DAY][0] == ICAL_RECURRENCE_ARRAY_MAX) {
+            /* Weekly recurrences with no BY_DAY data should occur on the
+               same day of the week as the start time . */
+            impl->by_ptrs[BY_DAY][0] = (short)get_day_of_week(impl);
+
+        } else {
+            adjust_to_byday(impl);
+
+            /* If start == DTSTART, adjust rstart */
+            if (icaltime_compare(start, impl->dtstart) == 0) {
+                impl->rstart = occurrence_as_icaltime(impl, 0);
+            }
+
+            /* Get (adjusted) start date as RSCALE date */
+            start = occurrence_as_icaltime(impl, 0);
+
+            if ((interval > 1) &&
+                (diff = (day_diff(impl, impl->rstart, start) + 6) / 7) % interval) {
+                /* Specified week doesn't match interval -
+                   bump start to next week that matches interval */
+                increment_monthday(impl, 7 * (interval - diff));
+            }
+        }
+        break;
+
+    case ICAL_DAILY_RECURRENCE:
+        if ((interval > 1) &&
+            (diff = day_diff(impl, impl->rstart, impl->istart) % interval)) {
+            /* Specified day doesn't match interval -
+               bump start to next day that matches interval */
+            increment_monthday(impl, interval - diff);
+        }
+        break;
+
+    default:
+        break;
+    }
+
+    /* Get start date as Gregorian date */
+    impl->last = occurrence_as_icaltime(impl, 1);
+
+    /* Fail if first instance exceeds MAX_TIME_T_YEAR */
+    if (impl->last.year > MAX_TIME_T_YEAR) {
+        icalerror_set_errno(ICAL_MALFORMEDDATA_ERROR);
+        return 0;
+    }
+
+    return 1;
+}
+
+int icalrecur_iterator_set_start(icalrecur_iterator *impl,
+                                 struct icaltimetype start)
+{
+    /* We can't adjust start date if we need to count occurrences */
+    if (impl->rule.count > 0) {
+        icalerror_set_errno(ICAL_MALFORMEDDATA_ERROR);
+        return 0;
+    }
+
+    /* Convert start to same time zone as DTSTART */
+    start = icaltime_convert_to_zone(start, (icaltimezone *) impl->dtstart.zone);
+
+    if (icaltime_compare(start, impl->dtstart) < 0) {
+        /* If start is before DTSTART, use DTSTART */
+        start = impl->dtstart;
+    }
+    else if (!icaltime_is_null_time(impl->rule.until) &&
+        icaltime_compare(start, impl->rule.until) > 0) {
+        /* If start is after UNTIL, we're done */
+        impl->last = start;
+        return 1;
+    }
+
+    return __iterator_set_start(impl, start);
+}
+
 /************************** Type Routines **********************/
 
 void icalrecurrencetype_clear(struct icalrecurrencetype *recur)
@@ -2868,7 +3098,7 @@ int icalrecurrencetype_month_month(short month)
  * back into local time before using them.
  */
 
-int icalrecur_expand_recurrence(char *rule,
+int icalrecur_expand_recurrence(const char *rule,
                                 time_t start, int count, time_t *array)
 {
     struct icalrecurrencetype recur;
@@ -2896,6 +3126,8 @@ int icalrecur_expand_recurrence(char *rule,
         }
         icalrecur_iterator_free(ritr);
     }
+    if(recur.rscale)
+        free(recur.rscale);
 
     return 1;
 }
