@@ -12,15 +12,15 @@
     icalheapset   Store components on the heap
     icalmysqlset  Store components in a mysql database.
 
- This program is free software; you can redistribute it and/or modify
+ This library is free software; you can redistribute it and/or modify
  it under the terms of either:
 
     The LGPL as published by the Free Software Foundation, version
     2.1, available at: http://www.gnu.org/licenses/lgpl-2.1.html
 
-  Or:
+ Or:
 
-    The Mozilla Public License Version 1.0. You may obtain a copy of
+    The Mozilla Public License Version 2.0. You may obtain a copy of
     the License at http://www.mozilla.org/MPL/
 
  The Original Code is eric. The Initial Developer of the Original
@@ -37,7 +37,7 @@
 #include "icalfileset.h"
 #include "icalfilesetimpl.h"
 
-#if defined(WITH_BDB)
+#if defined(HAVE_BDB)
 #include "icalbdbset.h"
 #include "icalbdbsetimpl.h"
 #endif
@@ -104,7 +104,7 @@ static icalset icalset_fileset_init = {
     NULL
 };
 
-#if defined(WITH_BDB)
+#if defined(HAVE_BDB)
 static icalset icalset_bdbset_init = {
     ICAL_BDB_SET,
     sizeof(icalbdbset),
@@ -145,6 +145,7 @@ static int load(const char *file)
 {
     void *modh;
     fptr inith;
+    char *dlerr;
     icalset *icalset_init_ptr;
 
     if ((modh = dlopen(file, RTLD_NOW)) == 0) {
@@ -152,8 +153,12 @@ static int load(const char *file)
         return 0;
     }
 
-    if ((inith = (fptr) dlsym(modh, "InitModule")) == 0) {
-        perror("dlsym");
+    (void)dlerror(); /* clear */
+    inith = (fptr)dlsym(modh, "InitModule");
+    dlerr = dlerror();
+    if (dlerr != NULL) {
+        fprintf(stderr, "dlsym error: %s\n", dlerr);
+        dlclose(modh);
         return 0;
     }
 
@@ -161,6 +166,8 @@ static int load(const char *file)
         pvl_push(icalset_kinds, &icalset_init_ptr);
     }
 
+    (void)dlerror(); /* clear */
+    dlclose(modh);
     return 1;
 }
 
@@ -211,7 +218,7 @@ static void icalset_init(void)
 
     pvl_push(icalset_kinds, &icalset_fileset_init);
     pvl_push(icalset_kinds, &icalset_dirset_init);
-#if defined(WITH_BDB)
+#if defined(HAVE_BDB)
     pvl_push(icalset_kinds, &icalset_bdb4set_init);
 #endif
 
@@ -244,7 +251,7 @@ icalset *icalset_new(icalset_kind kind, const char *dsn, void *options)
     }
 
     for (e = pvl_head(icalset_kinds); e != 0; e = pvl_next(e)) {
-        impl = (icalset *) pvl_data(e);
+        impl = (icalset *)pvl_data(e);
         if (impl->kind == kind) {
             break;
         }
@@ -254,8 +261,8 @@ icalset *icalset_new(icalset_kind kind, const char *dsn, void *options)
         return (NULL);
     }
 
-    data = (icalset *) malloc(impl->size);
-    if (data == (icalset *) NULL) {
+    data = (icalset *)malloc(impl->size);
+    if (data == (icalset *)NULL) {
         icalerror_set_errno(ICAL_NEWFAILED_ERROR);
         errno = ENOMEM;
         return 0;
@@ -269,8 +276,10 @@ icalset *icalset_new(icalset_kind kind, const char *dsn, void *options)
     data->dsn = strdup(dsn);
 #else
     switch (kind) {
-    case ICAL_FILE_SET:
-        data = (icalset *) malloc(sizeof(icalfileset));
+    case ICAL_FILE_SET: {
+        icalfileset *fdata;
+        fdata = (icalfileset *)malloc(sizeof(icalfileset));
+        data = (icalset *)fdata;
         if (data == 0) {
             icalerror_set_errno(ICAL_NEWFAILED_ERROR);
             errno = ENOMEM;
@@ -279,8 +288,11 @@ icalset *icalset_new(icalset_kind kind, const char *dsn, void *options)
         memset(data, 0, sizeof(icalfileset));
         *data = icalset_fileset_init;
         break;
-    case ICAL_DIR_SET:
-        data = (icalset *) malloc(sizeof(icaldirset));
+    }
+    case ICAL_DIR_SET: {
+        icaldirset *ddata;
+        ddata = (icaldirset *)malloc(sizeof(icaldirset));
+        data = (icalset *)ddata;
         if (data == 0) {
             icalerror_set_errno(ICAL_NEWFAILED_ERROR);
             errno = ENOMEM;
@@ -289,9 +301,12 @@ icalset *icalset_new(icalset_kind kind, const char *dsn, void *options)
         memset(data, 0, sizeof(icaldirset));
         *data = icalset_dirset_init;
         break;
-#if defined(WITH_BDB)
-    case ICAL_BDB_SET:
-        data = (icalset *) malloc(sizeof(icalbdbset));
+    }
+#if defined(HAVE_BDB)
+    case ICAL_BDB_SET: {
+        icalbdbset *bdata;
+        bdata = (icalbdbset *)malloc(sizeof(icalbdbset));
+        data = (icalset *)bdata;
         if (data == 0) {
             icalerror_set_errno(ICAL_NEWFAILED_ERROR);
             errno = ENOMEM;
@@ -300,6 +315,7 @@ icalset *icalset_new(icalset_kind kind, const char *dsn, void *options)
         memset(data, 0, sizeof(icalbdbset));
         *data = icalset_bdbset_init;
         break;
+    }
 #endif
 
     default:
