@@ -5,15 +5,16 @@
  (C) COPYRIGHT 2000 Eric Busboom <eric@softwarestudio.org>
       http://www.softwarestudio.org
 
- The contents of this file are subject to the Mozilla Public License
- Version 1.0 (the "License"); you may not use this file except in
- compliance with the License. You may obtain a copy of the License at
- http://www.mozilla.org/MPL/
+ This library is free software; you can redistribute it and/or modify
+ it under the terms of either:
 
- Software distributed under the License is distributed on an "AS IS"
- basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
- the License for the specific language governing rights and
- limitations under the License.
+    The LGPL as published by the Free Software Foundation, version
+    2.1, available at: http://www.gnu.org/licenses/lgpl-2.1.html
+
+ Or:
+
+    The Mozilla Public License Version 2.0. You may obtain a copy of
+    the License at http://www.mozilla.org/MPL/
 
  The Initial Developer of the Original Code is Eric Busboom
 ======================================================================*/
@@ -33,11 +34,12 @@
 #endif
 
 #define TMPSIZE 2048
+#define TMPSIZE_SMALL 128
 #define SENDMAIL "/usr/lib/sendmail -t"
 
 char *program_name;
 
-void usage(char *message);
+void usage(const char *message);
 
 enum options
 {
@@ -106,11 +108,16 @@ enum file_type test_file(char *path)
 
 char *lowercase(const char *str)
 {
+    char *new;
     char *p = 0;
-    char *new = strdup(str);
 
-    if (str == 0) {
-        return 0;
+    if (!str)
+        return NULL;
+
+    new = strdup(str);
+
+    if (!new) {
+        return NULL;
     }
 
     for (p = new; *p != 0; p++) {
@@ -139,14 +146,19 @@ icalcomponent *get_first_real_component(icalcomponent *comp)
 char *make_mime(const char *to, const char *from, const char *subject,
                 const char *text_message, const char *method, const char *ical_message)
 {
+    if ((to == NULL) || (from == NULL) || (subject == NULL) ||
+        (text_message == NULL) || (ical_message == NULL)) {
+        return NULL;
+    }
+
     size_t mess_size =
         strlen(to) +
         strlen(from) + strlen(subject) + strlen(text_message) + strlen(ical_message) + TMPSIZE;
 
     char mime_part_1[TMPSIZE];
     char mime_part_2[TMPSIZE];
-    char content_id[TMPSIZE];
-    char boundary[TMPSIZE];
+    char content_id[TMPSIZE_SMALL];
+    char boundary[TMPSIZE_SMALL];
     struct utsname uts;
     char *m;
 
@@ -158,15 +170,15 @@ char *make_mime(const char *to, const char *from, const char *subject,
     uname(&uts);
 
     srand((unsigned int)(time(0) << getpid()));
-    snprintf(content_id, TMPSIZE, "%d-%d@%s", (int)time(0), rand(), uts.nodename);
-    snprintf(boundary, TMPSIZE, "%d-%d-%s", (int)time(0), rand(), uts.nodename);
+    snprintf(content_id, TMPSIZE_SMALL, "%d-%d@%s", (int)time(0), rand(), uts.nodename);
+    snprintf(boundary, TMPSIZE_SMALL, "%d-%d-%s", (int)time(0), rand(), uts.nodename);
 //krazy:cond=style
     snprintf(mime_part_1, TMPSIZE, "Content-ID: %s\n\
 Content-type: text/plain\n\
 Content-Description: Text description of error message\n\n\
 %s\n\n--%s", content_id, text_message, boundary);
 
-    if (ical_message != 0 && method != 0) {
+    if (method != 0) {
         snprintf(mime_part_2, TMPSIZE, "Content-ID: %s\n\
 Content-type: text/calendar; method=%s\n\
 Content-Description: iCal component reply\n\n\
@@ -201,6 +213,7 @@ void return_failure(icalcomponent *comp, char *message, struct options_struct *o
     char *local_attendee = opt->calid;
     FILE *p;
     const char *org_addr;
+    char *mime;
 
     icalcomponent *inner = get_first_real_component(comp);
 
@@ -227,8 +240,12 @@ void return_failure(icalcomponent *comp, char *message, struct options_struct *o
         exit(1);
     }
 
-    fputs(make_mime(org_addr, local_attendee, "iMIP error",
-                    message, "reply", icalcomponent_as_ical_string(comp)), p);
+    mime = make_mime(org_addr, local_attendee, "iMIP error",
+                     message, "reply", icalcomponent_as_ical_string(comp));
+    if (mime) {
+        fputs(mime, p);
+        free(mime);
+    }
 
     if (opt->errors == ERRORS_TO_ORGANIZER) {
         pclose(p);
@@ -241,7 +258,11 @@ void return_error(icalcomponent *comp, char *message, struct options_struct *opt
     _unused(comp);
     _unused(opt);
 
-    fputs(make_mime("Dest", "Source", "iMIP system failure", message, 0, 0), stdout);
+    char *mime = make_mime("Dest", "Source", "iMIP system failure", message, "", "");
+    if (mime) {
+        fputs(mime, stdout);
+        free(mime);
+    }
 }
 
 icalcomponent *make_reply(icalcomponent *comp, icalproperty *return_status,
@@ -304,6 +325,9 @@ int check_attendee(icalproperty *p, struct options_struct *opt)
     const char *s = icalproperty_get_attendee(p);
     char *lower_attendee = lowercase(s);
     char *local_attendee = opt->calid;
+    char *start = lower_attendee;
+
+    int found = 0;
 
     /* Check that attendee begins with "mailto:" */
     if (strncmp(lower_attendee, "mailto:", 7) == 0) {
@@ -311,15 +335,12 @@ int check_attendee(icalproperty *p, struct options_struct *opt)
         lower_attendee += 7;
 
         if (strcmp(lower_attendee, local_attendee) == 0) {
-            return 1;
+            found = 1;
         }
-
-        lower_attendee -= 7;
-
-        free(lower_attendee);
     }
 
-    return 0;
+    free(start);
+    return found;
 }
 
 char static_component_error_str[MAXPATHLEN];
@@ -434,7 +455,7 @@ char *check_component(icalcomponent *comp, icalproperty ** return_status,
     return component_error_str;
 }
 
-void usage(char *message)
+void usage(const char *message)
 {
     _unused(message);
 //krazy:cond=style
@@ -614,20 +635,20 @@ void get_options(int argc, char *argv[], struct options_struct *opt)
                     fprintf(stderr,
                             "%s: Failed to create calendar directory %s: %s\n",
                             program_name, facspath, strerror(errno));
+                    free(facspath);
                     exit(1);
                 } else {
                     fprintf(stderr, "%s: Creating calendar directory %s\n", program_name, facspath);
-                    free(facspath);
                 }
 
             } else if (type == REGULAR || type == ERROR) {
                 fprintf(stderr, "%s: Cannot create calendar directory %s\n",
                         program_name, facspath);
+                free(facspath);
                 exit(1);
             }
-        } else {
-            free(facspath);
         }
+        free(facspath);
     }
 }
 
@@ -713,6 +734,7 @@ icalcomponent *read_nonmime_component(struct options_struct * opt)
         icalerror_set_error_state(ICAL_MALFORMEDDATA_ERROR, es);
 
         if (comp != 0) {
+            icalparser_free(parser);
             return comp;
         }
 
@@ -722,6 +744,7 @@ icalcomponent *read_nonmime_component(struct options_struct * opt)
         fclose(stream);
     }
 
+    icalparser_free(parser);
     return comp;
 }
 
